/*************************************************************************************************************************/
/* Description:
/* This program is used to generate Prime MST algorithm for undirected graphs
/* 

/* Release date of FLSv1.0: Aug.05, 2018
/* Latest updated date of FLSv2.0: Oct.02, 2018
/*************************************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "mst.h"
#include "util.h"
#include "file.h"
#include "brcut.h"

// create an undirected graph
int CreateGraph(MGraph *G, short *row, short *col, int *type, float *dist, 
				 const int len, const int xsize, const int ysize, const int cluster_limit, int key_type) {

	int size;
	int i, j, ka;
	short *key_row, *key_col;
	float *key_dist;
	float box_dist, inf_dist=(float)sqrt(xsize*xsize+ysize*ysize);

	for (size = 0,i = 0; i < len; ++i) {
		if (type[i]==key_type)
			++size;
	}

	if (size > cluster_limit) {
		return -1;
	}

	AllocateMemory(&key_row, size, sizeof(short));
	AllocateMemory(&key_col, size, sizeof(short));
	AllocateMemory(&key_dist, size, sizeof(float));

	// 
	for (ka = 0,i = 0; i < len; ++i) {
		if (type[i] == key_type) {
			key_row[ka] = row[i];
			key_col[ka] = col[i];
			key_dist[ka] = dist[i];
			++ka;
		}
	}

	// number of vertices and edges
	G->vexnum = size;
	G->arcnum = size*size;

	// 
	AllocateMemory(&(G->vexs), size, sizeof(int));
	AllocateMemory(&(G->arcs), size*size, sizeof(float));

	// 
	memset(G->arcs, 0, size*size*sizeof(float));
	for (i = 0; i < size; ++i) {
		G->vexs[i] = key_row[i]*ysize+key_col[i];
		for (j = i+1; j < size; ++j) {
			box_dist = (float)DIS(key_row[i], key_col[i], key_row[j], key_col[j]);
			G->arcs[i*size+j] = box_dist;
			G->arcs[j*size+i] = box_dist;
		}
	}

	//
	free(key_row);
	free(key_col);
	free(key_dist);
	return 1;
}

// locate the point u in the vertex matrix
int LocateVex(MGraph *G, int u) {

	int i;
	for (i = 0; i < G->vexnum; ++i) {
		if (G->vexs[i] == u) {
			return i;
		}
	}
	ErrorHandle("Error in locate in Graph", RUNTIME_ERROR);
	return -1;
}

int Minimum(struct MCloseEdge_tag *closedge, int len) {

	int i,k=0;
	float min = 10000.0f;

	for (i = 0; i < len; ++i) {
		if (closedge[i].lowcost > 0 && closedge[i].lowcost < min) {
			min = closedge[i].lowcost;
			k = i;
		}
	}
	return k;
}

//Prim MST
void MinSpanTreePrim(MGraph *G, int u, unsigned char *bitflags, const int xsize, const int ysize) {

	int i, j, k;
	int stp, si, sj, edp, ei, ej;
	int size = G->vexnum;
	CloseEdge closedge;

	AllocateMemory(&closedge, G->vexnum, sizeof(struct MCloseEdge_tag));
	bitflags[u] &= ~NEED_TODO;
	k = LocateVex(G, u);
	for (j = 0; j < G->vexnum; ++j) {
		if (j != k) {
			closedge[j].adjvex = u;
			closedge[j].lowcost = G->arcs[k*size+j];
		}
	}
	closedge[k].lowcost = 0;

	for (i = 1; i < G->vexnum; ++i) {
		//
		k = Minimum(closedge, G->vexnum);
		closedge[k].lowcost = 0;
		stp = closedge[k].adjvex;
		edp = G->vexs[k];
		si = stp/ysize;
		sj = stp%ysize;
		ei = edp/ysize;
		ej = edp%ysize;
		PlaceCut(bitflags, si, sj, ei, ej, xsize, ysize);
		bitflags[edp] &= ~NEED_TODO;

		//
		for (j = 0; j < G->vexnum; j++) {
			if (G->arcs[k*size+j]<closedge[j].lowcost) {
				closedge[j].adjvex = G->vexs[k];
				closedge[j].lowcost = G->arcs[k*size+j];
			}
		}
	}
	free(closedge);
}

//
void DestroyGraph(MGraph *G) {

	free(G->vexs);
	free(G->arcs);
}