/*************************************************************************************************************************/
/* Description:
/* This program is used to generate quality matrix for input interferogram.

/* Release date of FLSv1.0: Aug.05, 2018
/* Latest updated date of FLSv2.0: Oct.02, 2018
/*************************************************************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "qual.h"
#include "file.h"
#include "residue.h"

float Curve(float para) {
	if(para < -PI) return (float)(para+2*PI);
	else if(para > PI) return (float)(para-2*PI);
	else return para;
}

void GetQualMap(float *phase, float *qual, const int xsize, const int ysize) {
	
	int i, j, dx, dy;
	float delta1 = 0.0, delta2 = 0.0, delta11, delta22;
	if (qual == NULL) {
		fprintf(STDERR, "NULL pointer.\nAbort.\n");
		exit(RUNTIME_ERROR);
	}

	for (i = 0; i < xsize; i++) {
		fprintf(STDOUT, "\b\b\b\b%d%%", i*100/(xsize-1));
		for (j = 0; j < ysize; j++) {
			if (i>=3 && i<=(xsize-4) && j>=3 && j<=(ysize-4)) {
				for(dx = -2; dx <= 2; dx++) {
					for(dy = -2; dy <= 2; dy++) {
						delta1 += Curve(phase[(i+dx+1)*ysize+(j+dy)]-phase[(i+dx)*ysize+(j+dy)]);
						delta2 += Curve(phase[(i+dx)*ysize+(j+dy+1)]-phase[(i+dx)*ysize+(j+dy)]);
					}
				}
				delta1 = delta1/25;
				delta2 = delta2/25;
				delta11 = 0.0;
				delta22 = 0.0;
				for(dx = -2; dx <= 2; dx++) {
					for(dy = -2; dy <= 2; dy++) {
						delta11 += DOUBLE(Curve(phase[(i+dx+1)*ysize+(j+dy)]-phase[(i+dx)*ysize+(j+dy)])-delta1);
						delta22 += DOUBLE(Curve(phase[(i+dx)*ysize+(j+dy+1)]-phase[(i+dx)*ysize+(j+dy)])-delta2);
					}
				}
				delta11 = (float)(sqrt(delta11));
				delta22 = (float)(sqrt(delta22));
				qual[i*ysize+j] = (delta11+delta22)/25;
			}
			else {
				qual[i*ysize+j]=1.0;
			}
			delta1 = 0.0;
			delta2 = 0.0;
		}
	}

	//
	for(i = 0; i < xsize; i++) {
		for(j = 0; j < ysize; j++) {
			qual[i*ysize+j] = (float)((1-qual[i*ysize+j])+0.0000000001);
		}
	}
}